#include "CDF.h"
#include "params.h"

const port_t CDF::flux = 0;
const port_t CDF::soln_left = 1;
const port_t CDF::soln_right = 2;

CDF::CDF():
atomic(),
index(-1)
{}

CDF::CDF(int x, vec ul, vec ur):
atomic(),
ul(ul),
ur(ur),
index(x)
{}

CDF::CDF(const CDF& src):
atomic(src),
ul(src.ul),
ur(src.ur),
index(src.index)
{}

const CDF& CDF::operator=(const CDF& src)
{
	atomic::operator=(src);
	ul = src.ul;
	ur = src.ur;
	index = src.index;
	return *this;
}

void CDF::init()
{
	compute_flux();
	hold(0.0);
}

void CDF::delta_int()
{
	passivate();
}

void CDF::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	delta_conf(x);
}

void CDF::delta_conf(const adevs_bag<PortValue>& x)
{
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == soln_left)
			ul = x.get(i).value.get_value();
		else
			ur = x.get(i).value.get_value();
	}
	compute_flux();
	hold(0.0);
}

void CDF::output_func(adevs_bag<PortValue>& y)
{
	IOtype o(IOtype::CDF,index,f);
	output(flux,o,y);
}

void CDF::compute_flux() 
{ 
	f = fluxes(ul,ur); 
}
