#include "cell.h"

const port_t cell::flux_left = 0;
const port_t cell::flux_right = 1;
const port_t cell::soln = 2;
double cell::dx = 1.0;
double cell::T_min = INFINITY;
double cell::D[3] = { 0.0,0.0,0.0 };
		
cell::cell():
atomic(),
index(-1)
{}

cell::cell(int x, vec u0):
atomic(),
c(0),
index(x)
{
	for (unsigned i = 0; i < 3; i++)
	{
		q.setParams(i,D[i]);
		q.init(i,u0[i]);
	}
}

cell::cell(const cell& src):
atomic(src),
q(src.q),
fl(src.fl),
fr(src.fr),
c(src.c),
index(src.index)
{}

const cell& cell::operator=(const cell& src)
{
	atomic::operator=(src);
	q = src.q;
	fl = src.fl;
	fr = src.fr;
	c = src.c;
	index = src.index;
	return *this;
}

void cell::init()
{
	c = 0;
	T_last = 0.0;
	hold(q.ta());
}

void cell::delta_int()
{
	c++;
	if (timeCurrent()-T_last > 0.0)
		T_min = min(timeCurrent()-T_last,T_min);
	T_last = timeCurrent();
	q.integ(q.ta());
	hold(q.ta());
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	c++;
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == flux_left)
			fl = x.get(i).value.get_value();
		else
			fr = x.get(i).value.get_value();
	}
	q.integ(e,*((fl-fr)/dx));
	hold(q.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& x)
{
	c++;
	if (timeCurrent()-T_last > 0.0)
		T_min = min(timeCurrent()-T_last,T_min);
	T_last = timeCurrent();
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == flux_left)
			fl = x.get(i).value.get_value();
		else
			fr = x.get(i).value.get_value();
	}
	q.integ(q.ta(),*((fl-fr)/dx));
	hold(q.ta());
}

void cell::output_func(adevs_bag<PortValue>& y)
{
	vec y_(q.out(0),q.out(1),q.out(2));
	IOtype o(IOtype::CELL,index,y_);
	output(soln,o,y);
}

vec cell::getState() const
{
	vec s;
	for (int i = 0; i < 3; i++)
		s[i] = q.state(i);
	return s;
}
