#include "multicell.h"
#include "params.h"
#include <vector>
#include <iostream>
#include <fstream>
#include "perf_timer.h"
using namespace std;

static vector<vec> data;
static double tend;

void Initialize( void )
{
	int LEN;
	char str[80];
	/* loading data from file "classic/godunov.ini" */
	ifstream pF ("classic/godunov.ini");
	if(pF == 0) 
	{
		cerr << "Cannot open file godunov.ini" << endl;
		exit( -1 );
   }

   int i = 0;
	double deltaT;

   do
	{
		pF.getline(str,79);
		if (!pF.eof()) 
		{
			switch(i) 
			{
				case 0: LEN    = atoi( str ); break;
				case 1: cell::dx = atof( str ); break;
				case 2: deltaT = atof( str ); break;
				case 3: tend = atof( str ); break;
				case 4: cell::D[0] = atof( str ); break;
				case 5: cell::D[1] = atof( str ); break;
				case 6: cell::D[2] = atof( str ); break;
				default: break;
			} /* end switch() */
		i++;
		} /* end if() */
		else 
		{
			cerr << "Error while reading file godunov.ini" << endl;
			exit(1);
		}
	} 
	while( i < 7 );

	pF.close();

	int LENN = LEN+1;
	vec u;

	/* initial conditions */
	double U, R, P;
	for( i = 1, U = 0., R = 2., P = 200000.; i <= LEN/2; i++ ) 
	{
		u[0] = R;
		u[1] = R * U;
		u[2] = P / K_1 + 0.5 * R * U * U;
		data.push_back(u);
	} /* end for() */
   for( U = 0., R = 1., P = 100000.; i < LENN; i++ ) 
	{
		u[0] = R;
		u[1] = R * U;
		u[2] = P / K_1 + 0.5 * R * U * U;
		data.push_back(u);
	}

	/* left boundary conditions */
	u[0] = data[0].get(0);
	u[1] = -1.0 * data[0].get(1);
	u[2] = data[0].get(2);
	data.insert(data.begin(),u);
	/* right boundary conditions */
	u[0] = data[data.size()-1].get(0);
	u[1] = -1.0 * data[data.size()-1].get(1);
	u[2] = data[data.size()-1].get(2);
	data.push_back(u);
	
} /* end Initialize( ) */

int main(int argc, char** argv) 
{
	try 
	{
		Initialize();
		cout << "tend =  " << tend << endl;
		multicell space(data);
		devssim sim(&space);
		tic();
		sim.run(tend);
		double e = tocs();
		cout << e << ", T_min = " << cell::T_min << endl;
		space.report();
	}
	catch(adevs_exception& err) 
	{
		cerr << err.getErrorMsg() << endl;
	}
	return 0;
}
