#include "multicell.h"
#include "params.h"
#include <stdio.h>
#include <fstream>
using namespace std;

multicell::multicell(const vector<vec>& U):
devsn()
{
	nx = new space_manager(U,this);
}

multicell::multicell(const multicell& src):
devsn(src)
{
	nx = new space_manager(*(src.nx),this);
}

multicell::space_manager::space_manager(
const vector<vec>& U, devs* parent):
netExec(parent)
{
	unsigned index = 0;
	vector<vec>::const_iterator i = U.begin();
	vector<vec>::const_iterator k = U.begin(); k++;
	// left boundary and interior and right boundary
	for (; index < U.size()-1; index++,i++,k++)
	{
		cdfs.push_back(CDF(index,*i,*k));
		if (index < U.size()-2)
		{
			cells.push_back(cell(index,*k));
		}
	}
}

multicell::space_manager::space_manager(const space_manager& src, devs* parent):
netExec(src,parent),
cdfs(src.cdfs),
cells(src.cells)
{
}

void multicell::space_manager::getComponents(adevs_set<devs*>& c)
{
	unsigned i;
	for (i = 0; i < cdfs.size(); i++)
		c.add(&(cdfs[i]));
	for (i = 0; i < cells.size(); i++)
		c.add(&(cells[i]));
}

void multicell::space_manager::report()
{
	ofstream stream("data");
	vector<cell>::iterator k;
	for (k = cells.begin(); k != cells.end(); k++)
	{
		vec s = (*k).getState();
		double R = s[0];
		double U = s[1] / R;
		double P = (s[2] - 0.5 * R * U * U ) * K_1;
//		stream <<(c->getIndex()/2.0)*cell::dx<<" "<<P<<" "<<R<<" "<<U<< endl;
		stream <<((*k).getIndex()-0.5)*cell::dx<<" "<<s[0]<<" "
			<<s[1]<<" "<<s[2]<<" "<<(*k).getCompCount()<<endl;
	}
	stream.close();
}	

void multicell::space_manager::route(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r) 
{ 
	unsigned index = pv.value.get_index();
	if (pv.value.get_source() == IOtype::CDF)
	{
		if (index > 0)
			r.add(EventReceiver(&(cells[index-1]),cell::flux_right));
		if (index < cells.size())
			r.add(EventReceiver(&(cells[index]),cell::flux_left));
	}
	else if (pv.value.get_source() == IOtype::CELL)
	{
		r.add(EventReceiver(&(cdfs[index]),CDF::soln_right));
		r.add(EventReceiver(&(cdfs[index+1]),CDF::soln_left));
	}
}
