#ifndef __vec_h_
#define __vec_h_
#include <math.h>

class vec
{
	public:
		vec()
		{
			for (int i = 0; i < 3; i++) u[i] = 0.0;
		}
		vec(const vec& src)
		{
			for (int i = 0; i < 3; i++) u[i] = src.u[i];
		}
		const vec& operator=(const vec& src)
		{
			for (int i = 0; i < 3; i++) u[i] = src.u[i];
			return *this;
		}
		vec(double x, double y, double z)
		{
			u[0] = x;
			u[1] = y;
			u[2] = z;
		}
		vec(double v[3])
		{
			for (int i = 0; i < 3; i++) u[i] = v[i];
		}
		bool operator==(const vec& x) const
		{
			return (x.get(0) == u[0] && x.get(1) == u[1] && x.get(2) == u[2]);
		}
		bool operator!=(const vec& x) const
		{
			return !(*this == x);
		}
		double& operator[](int i) { return u[i]; }
		double get(int i) const { return u[i]; }
		double length1() const
		{
			double l = 0.0;
			for (int i = 0; i < 3; i++) 
				l += fabs(u[i]);
			return l;
		}
		double length2() const
		{
			double l = 0.0;
			for (int i = 0; i < 3; i++) 
				l += u[i]*u[i];
			return sqrt(l);
		}
		vec operator*(double x) const
		{
			return vec(x*u[0],x*u[1],x*u[2]);
		}
		vec operator/(double x) const
		{
			return vec(u[0]/x,u[1]/x,u[2]/x);
		}
		vec& operator+=(const vec& x)
		{
			for (int i = 0; i < 3; i++) u[i] += x.get(i);
			return *this;
		}
		vec& operator-=(const vec& x)
		{
			for (int i = 0; i < 3; i++) u[i] -= x.get(i);
			return *this;
		}
		vec operator-(const vec& x) const
		{
			return vec(u[0]-x.get(0),u[1]-x.get(1),u[2]-x.get(2));
		}
		vec operator+(const vec& x) const
		{
			return vec(u[0]+x.get(0),u[1]+x.get(1),u[2]+x.get(2));
		}
		void zero()
		{
			for (int i = 0; i < 3; i++) u[i] = 0.0;
		}
		const double* operator*()
		{
			return u;
		}
		~vec(){}
		friend vec operator*(double x,const vec& u)
		{
			return u*x;
		}
	private:
		double u[3];
};


#endif
