#include "cell.h"
#include "genr.h"
#include "segment.h"
#include "transducer.h"
#include <iostream>
#include <fstream>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
using namespace std;

int main (int argc, char** argv) 
{
	long seed, cells, procs;
	double speed_mean, entry_mean;
	double sample_time, running_time;
	ifstream params (argv[1]);
	if (params == 0) 
	{
		cerr << "Failed to load file" << endl;
		exit(-1);
	}
	params >> speed_mean;
	params >> entry_mean;
	params >> cells;
	params >> sample_time;
	params >> running_time;
	params >> seed;
	params >> procs;
	params.close ();

	try 
	{
		transducer* t = new transducer (sample_time);
		genr* g = new genr (entry_mean, speed_mean, seed);
		segment* s = new segment(cells);

		pdevs_sim* tsim = new pdevs_sim(t);
		pdevs_sim* gsim = new pdevs_sim(g);
		pdevs_coord* ssim = new pdevs_coord(s);
		double ts = 0.0;
		while (ts < running_time)
		{
			ts = min(ssim->timeNext(),min(tsim->timeNext(),gsim->timeNext()));
			if (tsim->timeNext() - ts < EPSILON)
				tsim->computeInputOutput();
			if (gsim->timeNext() - ts < EPSILON)
				gsim->computeInputOutput();
			if (ssim->timeNext() - ts < EPSILON)
				ssim->computeInputOutput();
			adevs_bag<PortValue> y;
			for (int i = 0; i < gsim->getOutput().getSize(); i++)
				y.add(PortValue(segment::in,gsim->getOutput().get(i).value));
			ssim->inject(y);
			y.removeAll();
			for (int i = 0; i < ssim->getOutput().getSize(); i++)
				if (ssim->getOutput().get(i).port == segment::stats)
					y.add(PortValue(transducer::sample,ssim->getOutput().get(i).value));
			tsim->inject(y);
			ssim->computeDeltaFunc(ts);
			tsim->computeDeltaFunc(ts);
			gsim->computeDeltaFunc(ts);
		}
		t->print_report();
		}
		catch (adevs_exception& err) 
		{
			cerr << err.getErrorMsg() << endl;
		}
	return 0;
}
