#include "cell.h"
#include "genr.h"
#include "segment.h"
#include "transducer.h"
#include <iostream>
#include <fstream>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
using namespace std;

int main (int argc, char** argv) 
{
	long seed, cells, procs;
	double speed_mean, entry_mean;
	double sample_time, running_time;
	ifstream params (argv[1]);
	if (params == 0) 
	{
		cerr << "Failed to load file" << endl;
		exit(-1);
	}
	params >> speed_mean;
	params >> entry_mean;
	params >> cells;
	params >> sample_time;
	params >> running_time;
	params >> seed;
	params >> procs;
	params.close ();

	try 
	{
		staticDigraph* model = new staticDigraph;
		transducer* t = new transducer (sample_time);
		model->add(t);
		genr* g = new genr (entry_mean, speed_mean, seed);
		model->add(g);
		int segments = procs - 2;
		segment** s = new segment* [segments];
		int cells_per_proc = div(cells,segments).quot;
		int leftover = div(cells,segments).rem;
		printf("%d cells per process with %d left over\n",cells_per_proc,leftover);
		for (int i = 0; i < segments; i++) 
		{
			if (i == segments-1)
				s[segments-1] = new segment(cells_per_proc+leftover,i*cells_per_proc);
			else
				s[i] = new segment (cells_per_proc,i*cells_per_proc);
			model->add(s[i]);
		}
		for (int i = 0; i < segments; i++) 
		{
			model->couple(s[i],s[i]->stats,t,t->sample);
			if (i < segments - 1) 
				model->couple(s[i],s[i]->out,s[i+1],s[i+1]->in);
		}
		model->couple(g,g->out,s[0],s[0]->in);
		devssim sim (model);
		sim.run (running_time);
		t->print_report();
		cout << "t = " << sim.timeLast() << ", tN = " << sim.timeNext() << endl;
	}
	catch (adevs_exception& err) 
	{
		cerr << err.getErrorMsg() << endl;
	}
return 0;
}
