#ifndef __transducer__
#define __transducer__
#include "cell.h"
#include <map>
#include <list>

/**
Periodically generates a line of pixels representing conjested
areas.  Conjested areas are shown in black.  Observed segment
runs from right (cell 0) to left (last cell)
*/
class transducer: public atomic 
{
	public:
		static const port_t sample;
		transducer():atomic(){}
		transducer(stime_t t_sample):
		atomic(),mSampleTime(t_sample){}
		transducer(const transducer& src);
		const transducer& operator=(const transducer& src);
		object* clone() const { return new transducer(*this); }
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y){}
		void gc_output(adevs_bag<PortValue>& g){}
		void print_report();
		~transducer(){}
	private:
		std::map<int,int> cell_size;
		std::list<std::list<char> > display_lines;
		int inputs;
		double mSampleTime;
	};

#endif
