/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <pdevs.h>

void pdevs_sim::detachModel()
{
	model->gc_output(y);
	x.removeAll();
	y.removeAll();
	model->setAttachedSimulator(NULL);
	model = NULL;
	setModel(model);
}

void pdevs_sim::attachModel(atomic* d)
{
	if (model != NULL)
		THROW_BASE_EXCEPTION(
		"Did not detach model from simulator before attaching new model");
	model = d;
	model->setAttachedSimulator(this);
	setModel(model);
	x.removeAll();
	y.removeAll();
}

void pdevs_sim::computeInputOutput()
{
	TIME_TYPE t = model->timeCurrent();
	model->setTimeCurrent(model->timeNext());
	model->output_func(y);
	model->setTimeCurrent(t);
}
	
void pdevs_sim::computeDeltaFunc(TIME_TYPE t)
{
	model->setTimeCurrent(t);
	if (model->sigma() <= granule && x.getSize() == 0)
	{
		model->delta_int();
		model->setTimeLast(t);
	}
	else if (model->sigma() <= granule && x.getSize() > 0)
	{
		model->delta_conf(x);
		model->setTimeLast(t);
	}
	else if (model->sigma() > granule && x.getSize() > 0)
	{
		model->delta_ext(model->elapsed(),x);
		model->setTimeLast(t);
	}
	x.removeAll();
}

void pdevs_sim::endOfCycle()
{
	model->gc_output(y);
	y.removeAll();
}
