/***************
Copyright (C) 2001, 2002 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs.h>
#include <math.h>
#ifndef PI
#define PI 3.141592654
#endif

rv::rv(unsigned long seed) 
{
	_impl = new mtrand(seed);
}

rv::rv(random_seq* impl)
{
	_impl = impl;
}

rv::rv(const rv& src)
{
	_impl = src._impl->copy();
}

const rv& rv::operator=(const rv& src)
{
	delete _impl;
	_impl = src._impl->copy();
	return *this;
}

void rv::set_seed(unsigned long seed)
{
	_impl->set_seed (seed);
}

double rv::triangular(double a, double b, double c)
{
	double x = _impl->next_dbl ();
	if (x <= 0.5) return (2.0*(b-a)*x + a);
	return (2.0*(b-c)*(x-0.5) + c);
}

double rv::uniform(double a, double b) 
{
	return (_impl->next_dbl() * (b - a) + a);
}

double rv::exp(double m) 
{
	return (-m * log (_impl->next_dbl ()));
}

double rv::normal(double m, double s) 
{
	return m + (sqrt (-2.0 * log (_impl->next_dbl())) * 
		cos (PI * (2.0 * _impl->next_dbl() - 1.0))) * s;
}

rv::~rv() { delete _impl; }
