/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <xdevssim.h>

void xdevssim::initialize(TIME_TYPE t0)
{
	IO_up_to_date = false;
	input.removeAll();
	model->setTimeCurrent(t0);
	model->setTimeLast(t0);
	model->init();
	if (sim != NULL) delete sim;
	if (model->getType() == devs::ATOMIC)
		sim = new pdevs_sim((atomic*)model);
	else
		sim = new pdevs_coord((devsn*)model);
}

const adevs_bag<PortValue>& xdevssim::computeInputOutput()
{
	if (!IO_up_to_date)
	{
		sim->computeInputOutput();
		IO_up_to_date = true;
	}
	return sim->getOutput();
}
	
void xdevssim::deltFunc(TIME_TYPE t)
{
	if (!IO_up_to_date && (sim->timeNext() - t <= getAdevsTimeGranule())) 
		sim->computeInputOutput();
	sim->inject(input);
	sim->computeDeltaFunc(t);
	sim->endOfCycle();
	input.removeAll();
	IO_up_to_date = false;
}

devs* xdevssim::attachModel(devs* model_)
{
	devs* last_model = model;
	if (model_->getType() != model->getType())
		THROW_BASE_EXCEPTION("Model type mismatch");
	IO_up_to_date = false;
	input.removeAll();
	model = model_;
	sim->detachModel();
	if (model->getType() == devs::ATOMIC)
	{
		pdevs_sim* sim_ = (pdevs_sim*)sim;
		sim_->attachModel((atomic*)model);
	}
	else
	{
		pdevs_coord* sim_ = (pdevs_coord*)sim;
		sim_->attachModel((devsn*)model);
	}
	return last_model;
}

xdevssim::~xdevssim()
{
	if (sim != NULL)
		delete sim;
}
