#include <adevs.h>
#include <iostream>
#include <list>
#ifndef __delay_h_
#define __delay_h_

class delay: public atomic 
{
	public:
		static const port_t in;
		static const port_t out;

		delay (stime_t t):atomic(),dt(t){}
		delay(const delay& src):
		atomic(src),
		dt(src.dt),
		q(src.q){}
		object* clone() const { return new delay(*this); }
		void init()
		{
			q.clear();
			passivate();
		}
		void delta_int()
		{
			delay_q::iterator i;
			for (i = q.begin(); i != q.end(); i++)
				(*i).first -= ta();
			while (!q.empty())
			{
				if (q.front().first <= EPSILON)
				{
					delete q.front().second;
					q.pop_front();
				}
				else
				{
					assert(q.front().first >= 0.0);
					hold(q.front().first);
					return;
				}
			}
			passivate();
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			delay_q::iterator i;
			for (i = q.begin(); i != q.end(); i++)
				(*i).first -= e;
			for (int i = 0; i < x.getSize(); i++)
			{
				assert(x.get(i).port == in);
				q.push_back(pair<stime_t,object*>(dt,new object(*(x.get(i).value))));
			}
			assert(q.front().first >= 0.0);
			hold(q.front().first);
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			delta_int();
			delta_ext(0.0,x);
		}
		void output_func (adevs_bag<PortValue>& y)
		{
			delay_q::iterator i;
			for (i = q.begin(); i != q.end(); i++)
			{
				if ((*i).first <= ta());
					output(out,new object(*((*i).second)),y);
			}
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
				delete g[i].value;
		}
		~delay(){}

	private:	
		stime_t dt;
		typedef std::list<std::pair<stime_t,object*> > delay_q;
		delay_q q;
};

const port_t delay::in(0);
const port_t delay::out(1);

#endif
