#ifndef __gcd_h_
#define __gcd_h_
#include "genr.h"
#include "counter.h"
#include "delay.h"

class gcd: public staticDigraph 
{
	public:
		static const port_t in;
		static const port_t out;
		static const port_t signal;
		static const port_t start;
		static const port_t stop;

		gcd(const std::vector<stime_t>& pattern, stime_t dt,
		int iterations, bool active = false):staticDigraph()
		{
			genr* g = new genr(pattern,iterations,active);
			delay* d = new delay(dt);
			counter* c = new counter;
			build(g,c,d);
		}
		gcd(stime_t period, stime_t dt, 
		int iterations, bool active = false):staticDigraph()
		{
			genr* g = new genr(period,iterations,active);
			delay* d = new delay(dt);
			counter* c = new counter;
			build(g,c,d);
		}
		gcd(const gcd& src):staticDigraph(src){}
		object* clone() const { return new gcd(*this); }
		~gcd(){}
	private:
		void build(genr* g, counter* c, delay* d) 
		{
			add(g);
			add(d);
			add(c);
			couple(this,in,d,d->in);
			couple(this,start,g,g->start);
			couple(this,stop,g,g->stop);
			couple(g,g->signal,this,signal);
			couple(d,d->out,this,out);
			couple(d,d->out,c,c->in);
		}
};

const port_t gcd::in(0);
const port_t gcd::out(1);
const port_t gcd::start(2);
const port_t gcd::stop(3);
const port_t gcd::signal(4);

#endif
